/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
/*
Qt 4 Port by Rainer Strobel

replaced Qt 3 iterators through Qt 4 iterators
in method writeStateProcess:
replaced QList through Q3PtrList for compatibility
*/

#ifndef EXPORTVHDL_H
#define EXPORTVHDL_H

#include "Export.h"

class Options;

/**
 * @class ExportVHDL
 * @brief Class that manages export to VHDL.
 *
 */

class ExportVHDL : public Export
{
  public:
    ExportVHDL(Options* opt);

    void doExport();
    QString fileFilter();

  private:
    /// Name of the machine to export
    QString mname;
    /// If TRUE, the fixed encoding for the states is used, otherwise an enum type.
    bool state_encoding;
    /// If TRUE, the names of the inputs/outputs are used, otherwise the vectors a,o,q
    bool io_names;
    /// If TRUE, the reset signal will be handled synchronously, otherwise asynchronously.
    bool synchronous_reset;
    /// If TRUE, the negated reset signal will be used
    bool neg_reset;
    /// If TRUE, 'std_logic' is used instead of 'bit'
    bool use_std_logic;
    /// If TRUE, an I/O description is written in the header
    bool io_header;
    /// If TRUE, produce Alliance compliant code
    bool alliance;
    /// 0: IF-THEN condition notation,  1: CASE-WHEN notation
    int cond_notation;

    void writeEnvironment();
    void writeEntity();
    void writeArchitecture();
    void writeClockProcess();
    void writeStateProcess();
    void writeDescription();
#ifdef FHA
    void writeHeader(QString,QString);
#endif
};

#endif
