/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <qstring.h>
#include <q3valuelist.h>
//Added by qt3to4:
#include <Q3PtrList>

#include "TransitionInfo.h"
#include "IOInfoBin.h"
#include "IOInfoList.h"
#include "IOInfo.h"
//#include "IOInfoASCII.h"


/// Constructor
IOInfo::IOInfo(IOType t, bool inv/*=FALSE*/)
{
  type = t;
  role = FALSE;
  invert = inv;
  range = FALSE;
  any = FALSE;
  def = FALSE;
}

/// Destructor
IOInfo::~IOInfo()
{
}


/// Returns TRUE if @a io equals this object
bool IOInfo::equals(IOInfo* io)
{
  if(io==NULL)
    return FALSE;
  if (io->type!=type)
    return FALSE;
  if (io->isInverted()!=invert)
    return FALSE;
  if ((io->any && any)==TRUE)
    return TRUE;

  IOInfoBin b1(type), b2(type);
  int len1, len2, maxlen;
  len1 = getLength();
  len2 = io->getLength();

  maxlen = len1>len2? len1 : len2;
  
  qDebug("io length: "+QString::number(io->getLength())+" this length: "+QString::number(getLength()));

  b1 = convertToBin(maxlen);
  b2 = io->convertToBin(maxlen);

  QString s1, s2;
  s1 = b1.convertToBinStr();
  s2 = b2.convertToBinStr();

  if (s1==s2)
  {
    qDebug("equal : "+s1+" "+s2);
    return TRUE;
  }
  else
    return FALSE;
}


/// Returns TRUE if @a io matches this object (i.e. this I/O info contains @a io)
bool IOInfo::matches(IOInfo* io)
{
  IOInfoList list1;
  IOInfoList list2;
//  list1.setAutoDelete(TRUE);
//  list2.setAutoDelete(TRUE);

  if ((any && io->getAnyInput()))
    return TRUE;

  convertToBinList(list1);
  io->convertToBinList(list2);

  if (list1.isEmpty() || list2.isEmpty())
    return FALSE;
 

  /*
  // for debugging
  unsigned char buf[100];
  int len;
  QListIterator<IOInfo> it1(list1);
  for(;it1.current(); ++it1)
  {
    it1.current()->convertToASCII(buf, 100, len);
    qDebug("matches1: %s %c", it1.current()->convertToString().latin1(), buf[0]);
  }
  */

  QListIterator<IOInfo*> it1(list1);
  QListIterator<IOInfo*> it2(list2);
  /*
  if (invert)
  {
    for(;it.current(); ++it)
    {
      if (list1.contains(it.current()))
	return FALSE;
    }
    return TRUE;
  }
  else
  */
  {

    for(int c=0;it2.hasNext();c++)
    {


      if (!list1.contains(it2.next()))
        return FALSE;
    }
    
/*
    bool equal;
    for(;it2.hasNext();)
    {
      equal=FALSE;
      for(;it1.hasNext();)
      {
        if (it1.peekNext()->equals(it2.peekNext()))
	      equal=TRUE;
        it1.next();
      }
      if(equal==FALSE)
        return FALSE;
      it2.next();
    }
    */
    return TRUE;
  }
}


void IOInfo::expandList(IOInfoList& input, IOInfoList& list, bool resolve_invert)
{
  // creating eventlist (with events that could change a state) using all input conditions
  IOInfoList eventlist;
//  eventlist.setAutoDelete(TRUE);
  IOInfo *io;
  IOInfo *start, *end;
  IOInfo *new_start, *new_end;
  QMutableListIterator<IOInfo*> inputit(input);
  for(; inputit.hasNext();)
  {
    IOInfoList iolist;
//    iolist.setAutoDelete(TRUE);
    inputit.next()->split(iolist, resolve_invert);

    /*
    if (inputit.current()->isInverted())
    qDebug("NOT %s\n", inputit.current()->convertToString().latin1());
    else
    qDebug("%s\n", inputit.current()->convertToString().latin1());
    */

    QMutableListIterator<IOInfo*> it(iolist);
    for(; it.hasNext();)
    {
      io = it.next();
      qDebug("first loop: "+io->convertToString());
      //qDebug("io: %s", io->convertToString().latin1());

      QList<IOInfo*> list;
 //     list.setAutoDelete(TRUE);
      if (resolve_invert || !io->isInverted())
      {
        list = io->getSinglesInversion();

        if (!list.isEmpty())
        {
          start = list.front();
          end = list.back();

          new_start = start->clone();
          new_end = end->clone();
          new_start->setRole(TRUE);
          new_end->setRole(FALSE);

          eventlist.append(new_start);
          eventlist.append(new_end);
        }
      }
      else eventlist.append(io->clone());
    }
  }

  // creating output while traversing eventlist (sort of line sweep algorithm)

  int open_counter=0;  // open ranges
  //int prev_open;
  IOInfo* previo=NULL;
  IOInfo *io_minus1=NULL, *io_plus1=NULL;
  QString s, e;
  
//  eventlist.sort();
  qSort(eventlist.begin(), eventlist.end(), IOInfoList::lessThan);
  //qHeapSort(eventlist.first(), eventlist.last());
  //qHeapSort(eventlist);
  start=end=NULL;
  io = NULL;
  QMutableListIterator<IOInfo*> evit(eventlist);
  
  /*
  for(; evit.hasNext();)
  {
    io = evit.next();
    qDebug("second loop:"+io->convertToString());
    if(!list.contains(io))
      list.append(io);
  }
  */
  for(; evit.hasNext();)
  {
    previo = io;
    io = evit.next();
    qDebug("second loop:"+io->convertToString());
    if (io->isInverted())
    {
      if (!list.contains(io))
      {
        list.append(io->clone());
      }
    }
    else // not inverted
    {
      if (io->getRole()==TRUE)	// start event
      {
        if (open_counter==0)
          start = io;
        else
        {
          io_minus1 = io->getMinus1();
          if (io_minus1==NULL)
          {
            open_counter++;
            continue;
          }
          s = start->convertToString();
          e = io_minus1->convertToString();
          if (eventlist.compareItems(start, io_minus1)<=0)
          {
            IOInfo* new_io = start->clone();
            new_io->setRangeInfo(start, io_minus1);
            if (!list.contains(new_io))
            {
              list.append(new_io);
            }
          }

          delete io_minus1;
	  
          start = io;
        }
        open_counter++;
      }
      else			// end event
      {
        open_counter--;
        if (!io->equals(previo) || (io->equals(previo) 
             && previo!=NULL && io->getRole()!=previo->getRole()))
          end = io;

        if (start && end)
        {
          s = start->convertToString();
          e = end->convertToString();
          if (eventlist.compareItems(start, end)<=0)
          {
            IOInfo* new_io = start->clone();
            new_io->setRangeInfo(start, end);
            if (!list.contains(new_io))
            {
              list.append(new_io);
            }
          }
        }
        if (open_counter>0)
        {
          if (io_plus1)
            delete io_plus1;
          io_plus1 = io->getPlus1();
          start = io_plus1;
          if (io_plus1==NULL)
            return;
        }
      } // end if start/end
    } // end not inverted
  }
}


