/*
Copyright (C) 2000,2001 Stefan Duffner 

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include <qradiobutton.h>
#include <q3buttongroup.h>

#include "TransitionPropertiesDlgImpl.h"

#include "Transition.h"
#include "Error.h"
#include "TransitionInfo.h"
#include "Const.h"


/** 
 *  Constructs a TransitionPropertiesDlgImpl which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f' 
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
TransitionPropertiesDlgImpl::TransitionPropertiesDlgImpl( QWidget* parent,  const char* name, bool modal, Qt::WFlags fl )
    : TransitionPropertiesDlg( parent, name, modal, fl )
{
}

/**  
 *  Destroys the object and frees any allocated resources
 */
TransitionPropertiesDlgImpl::~TransitionPropertiesDlgImpl()
{
}


/// Validates the strings entered as the transition inputs and outputs
void TransitionPropertiesDlgImpl::validate()
{
  Error err;
  QString in, out;
  int type;
  int cres;

  in = le_input->text();
  out = le_output->text();
  type = getType();

  cres = Transition::conditionValid(type, in);
  if (cres)
  {
    switch (cres)
    {
      case 1:
	err.info(tr("Input condition is not in binary format."));
	break;
      case 2:
	err.info(tr("Incomplete escape sequence."));
	break;
      case 3:
	err.info(tr("'-' has to define a range."));
	break;
      case 4:
	err.info(tr("Output must be only one character."));
	break;
      case 5:
	err.info(tr("Invalid escape sequence.\nThe format is \\0xx, where xx is a\nhexadecimal value"));
        break;
      default:
	err.info(tr("Input condition is not in the correct format."));
	break;
    }
    return;
  }
  else
  {
    cres = Transition::conditionValid(type, out, FALSE);
    if (cres)
    {
      switch (cres)
      {
	case 1:
	  err.info(tr("Output condition is not in binary format."));
	  break;
	case 2:
	  err.info(tr("Incomplete escape sequence."));
	  break;
	case 3:
	  err.info(tr("'-' has to define a range."));
	  break;
	case 4:
	  err.info(tr("Output must be only one character."));
	  break;
	case 5:
	  err.info(tr("Invalid escape sequence.\nThe format is \\0xx, where xx is a\nhexadecimal value"));
	  break;
	default:
	  err.info(tr("Output condition is not in the correct format."));
	  break;
      }
      return;
    }
    else
      accept();
  }
}

/// Returns the type of the transition
int TransitionPropertiesDlgImpl::getType()
{
  if (rb_ascii->isChecked()) 
	return 1;
  if (rb_text->isChecked()) 
	return 2;
  else return 0;

//  return bg_type->selectedId();

  /*
  if (rb_ascii->isChecked())
    return Ascii;
  else
    return Binary;
    */
}

void TransitionPropertiesDlgImpl::resetFields()
{
  cb_default->setEnabled(FALSE);
  cb_invert->setEnabled(FALSE);
  cb_any->setEnabled(FALSE);
  le_input->setEnabled(TRUE);
  le_input->clear();
}


/// Sets the type of the transition
void TransitionPropertiesDlgImpl::setType(int t)
{
  if (t==Binary)
  {
    rb_bin->setEnabled(TRUE);
    rb_ascii->setEnabled(TRUE);
    rb_text->setEnabled(FALSE);

    rb_bin->setChecked(TRUE);

    cb_default->setEnabled(FALSE);
    cb_default->setChecked(FALSE);
    defaultClicked();
  }
  if (t==Ascii)
  {
    rb_bin->setEnabled(TRUE);
    rb_ascii->setEnabled(TRUE);
    rb_text->setEnabled(FALSE);

    rb_ascii->setChecked(TRUE);
    cb_default->setEnabled(TRUE);
    defaultClicked();
  }
  else if (t==Text)
  {
    rb_bin->setEnabled(FALSE);
    rb_ascii->setEnabled(FALSE);
    rb_text->setEnabled(TRUE);

    rb_text->setChecked(TRUE);
    cb_default->setEnabled(false);
    cb_default->setChecked(false);
    cb_any->setEnabled(false);
    cb_any->setChecked(false);
    cb_invert->setEnabled(false);
    cb_invert->setChecked(false);
    //defaultClicked();
  }
}


/// Called when the 'Binary' button is clicked
void TransitionPropertiesDlgImpl::binaryClicked()
{
  le_input->setMaxLength(binmax_in);
  le_output->setMaxLength(binmax_out);

  cb_default->setEnabled(FALSE);
  cb_default->setChecked(FALSE);
  defaultClicked();
  anyClicked();
  invertClicked();
}

/// Called when the 'ASCII' button is clicked
void TransitionPropertiesDlgImpl::asciiClicked()
{
  le_input->setMaxLength(MAX_ASCII_INPUTLENGTH);
  le_output->setMaxLength(MAX_ASCII_OUTPUTLENGTH);

  if (!cb_any->isChecked() && !cb_invert->isChecked())
    cb_default->setEnabled(TRUE);
  else
    cb_default->setEnabled(FALSE);
  defaultClicked();
  anyClicked();
  invertClicked();
}

void TransitionPropertiesDlgImpl::freeTextClicked()
{
  le_input->setMaxLength(MAX_TEXT_INPUTLENGTH);
  le_output->setMaxLength(MAX_TEXT_OUTPUTLENGTH);

  cb_default->setEnabled(false);
  cb_default->setChecked(false);
  cb_any->setEnabled(false);
  cb_any->setChecked(false);
  cb_invert->setEnabled(false);
  cb_invert->setChecked(false);
    /*
  if (!cb_any->isChecked() && !cb_invert->isChecked())
    cb_default->setEnabled(TRUE);
  else
    cb_default->setEnabled(FALSE);
  defaultClicked();
  anyClicked();
  invertClicked();
  */
}


void TransitionPropertiesDlgImpl::anyClicked()
{
  if (cb_any->isChecked())
  {
    cb_invert->setEnabled(FALSE);
    cb_default->setEnabled(FALSE);
    le_input->setEnabled(FALSE);
  }
  else
  {
    cb_invert->setEnabled(TRUE);
    if (rb_ascii->isChecked() || rb_text->isChecked())
      cb_default->setEnabled(TRUE);
    le_input->setEnabled(TRUE);
  }
}

void TransitionPropertiesDlgImpl::defaultClicked()
{
  if (cb_default->isChecked())
  {
    cb_invert->setEnabled(FALSE);
    cb_any->setEnabled(FALSE);
    le_input->setEnabled(FALSE);
  }
  else
  {
    cb_invert->setEnabled(TRUE);
    cb_any->setEnabled(TRUE);
    le_input->setEnabled(TRUE);
  }
}

void TransitionPropertiesDlgImpl::invertClicked()
{
  if (cb_invert->isChecked())
  {
    cb_default->setEnabled(FALSE);
    cb_any->setEnabled(FALSE);
  }
  else
  {
    if (!cb_any->isChecked() && (rb_ascii->isChecked() || rb_text->isChecked()))
      cb_default->setEnabled(TRUE);
    cb_any->setEnabled(TRUE);
  }
}


